@@ -1,23 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2003-2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/xfs.h>
-#include <xfs/command.h>
+#include "platform_defs.h"
+#include "command.h"
 #include "init.h"
 #include "io.h"
 
@@ -31,6 +19,7 @@ fsync_f(
 {
 	if (fsync(file->fd) < 0) {
 		perror("fsync");
+		exitcode = 1;
 		return 0;
 	}
 	return 0;
@@ -43,6 +32,7 @@ fdatasync_f(
 {
 	if (fdatasync(file->fd) < 0) {
 		perror("fdatasync");
+		exitcode = 1;
 		return 0;
 	}
 	return 0;
@@ -51,15 +41,15 @@ fdatasync_f(
 void
 fsync_init(void)
 {
-	fsync_cmd.name = _("fsync");
-	fsync_cmd.altname = _("s");
+	fsync_cmd.name = "fsync";
+	fsync_cmd.altname = "s";
 	fsync_cmd.cfunc = fsync_f;
 	fsync_cmd.flags = CMD_NOMAP_OK | CMD_FOREIGN_OK;
 	fsync_cmd.oneline =
 		_("calls fsync(2) to flush all in-core file state to disk");
 
-	fdatasync_cmd.name = _("fdatasync");
-	fdatasync_cmd.altname = _("ds");
+	fdatasync_cmd.name = "fdatasync";
+	fdatasync_cmd.altname = "ds";
 	fdatasync_cmd.cfunc = fdatasync_f;
 	fdatasync_cmd.flags = CMD_NOMAP_OK | CMD_FOREIGN_OK;
 	fdatasync_cmd.oneline =
