@@ -499,8 +499,7 @@ mk_rbmino(xfs_mount_t *mp)
 
 	VFS_I(ip)->i_mode = S_IFREG;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
-	if (ip->i_afp)
-		ip->i_afp->if_format = XFS_DINODE_FMT_EXTENTS;
+	libxfs_ifork_zap_attr(ip);
 
 	set_nlink(VFS_I(ip), 1);	/* account for sb ptr */
 
@@ -739,8 +738,7 @@ mk_rsumino(xfs_mount_t *mp)
 
 	VFS_I(ip)->i_mode = S_IFREG;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
-	if (ip->i_afp)
-		ip->i_afp->if_format = XFS_DINODE_FMT_EXTENTS;
+	libxfs_ifork_zap_attr(ip);
 
 	set_nlink(VFS_I(ip), 1);	/* account for sb ptr */
 
@@ -838,8 +836,7 @@ mk_root_dir(xfs_mount_t *mp)
 
 	VFS_I(ip)->i_mode = mode|S_IFDIR;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
-	if (ip->i_afp)
-		ip->i_afp->if_format = XFS_DINODE_FMT_EXTENTS;
+	libxfs_ifork_zap_attr(ip);
 
 	set_nlink(VFS_I(ip), 2);	/* account for . and .. */
 
@@ -1209,7 +1206,7 @@ dir_binval(
 		return 0;
 
 	geo = tp->t_mountp->m_dir_geo;
-	ifp = XFS_IFORK_PTR(ip, XFS_DATA_FORK);
+	ifp = xfs_ifork_ptr(ip, XFS_DATA_FORK);
 	for_each_xfs_iext(ifp, &icur, &rec) {
 		for (dabno = roundup(rec.br_startoff, geo->fsbcount);
 		     dabno < rec.br_startoff + rec.br_blockcount;
